'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var _a;
var dpr = window.devicePixelRatio || 1;
(function (HIGHLIGHT_STATUS) {
    HIGHLIGHT_STATUS[HIGHLIGHT_STATUS["NONE"] = 0] = "NONE";
    HIGHLIGHT_STATUS[HIGHLIGHT_STATUS["BAD"] = 1] = "BAD";
    HIGHLIGHT_STATUS[HIGHLIGHT_STATUS["NORMAL"] = 2] = "NORMAL";
    HIGHLIGHT_STATUS[HIGHLIGHT_STATUS["GOOD"] = 3] = "GOOD";
})(exports.HIGHLIGHT_STATUS || (exports.HIGHLIGHT_STATUS = {}));
var DEFAULT_STYLES = {
    lineHeight: 24,
    fontSize: 16,
    lineWidth: 2,
    font: '16px sans-serif',
};
var DEFAULT_COLOR_MAP = (_a = {},
    _a[exports.HIGHLIGHT_STATUS.NONE] = 'rgba(60, 60, 67, 0.6)',
    _a[exports.HIGHLIGHT_STATUS.NORMAL] = '#000',
    _a[exports.HIGHLIGHT_STATUS.BAD] = '#fa4838',
    _a[exports.HIGHLIGHT_STATUS.GOOD] = '#26d07c',
    _a);
var Highlighter = (function () {
    function Highlighter(container, options) {
        this.styles = DEFAULT_STYLES;
        this.colorMap = DEFAULT_COLOR_MAP;
        this.highlightInfo = [];
        this.lines = 0;
        if (container instanceof HTMLCanvasElement) {
            this.canvas = container;
        }
        else {
            this.canvas = document.querySelector(container);
        }
        if (!this.canvas) {
            throw new Error('未获取到canvas标签！');
        }
        this.canvas.width = options.width;
        this.canvasWidth = options.width;
        this.styles = Object.assign({}, DEFAULT_STYLES, options === null || options === void 0 ? void 0 : options.styles);
        this.colorMap = Object.assign({}, DEFAULT_COLOR_MAP, options === null || options === void 0 ? void 0 : options.colorMap);
        this.ctx = this.canvas.getContext('2d');
    }
    Highlighter.prototype.calcPosition = function (text) {
        if (!(this.canvas && this.ctx))
            return;
        var _a = this.styles, font = _a.font, lineHeight = _a.lineHeight;
        var top = 0;
        var left = 0;
        this.ctx.font = font;
        var wordsArr = text.split(' ').map(function (item) { return item + ' '; });
        var line = '';
        for (var n = 0; n < wordsArr.length; n++) {
            var offsetX = this.ctx.measureText(line).width;
            var wordWidth = this.ctx.measureText(wordsArr[n]).width;
            var wordInfo = {
                status: exports.HIGHLIGHT_STATUS.NONE,
                underline: false,
                width: wordWidth,
                height: lineHeight,
                index: n,
                content: wordsArr[n],
            };
            if (offsetX + wordWidth > this.canvasWidth && n > 0) {
                this.lines++;
                line = wordsArr[n];
                top += lineHeight;
                left = 0;
            }
            else {
                line = line + wordsArr[n];
                left = offsetX;
            }
            this.highlightInfo.push(__assign(__assign({}, wordInfo), { left: left,
                top: top }));
        }
        if (line)
            this.lines++;
    };
    Highlighter.prototype.resetSize = function () {
        var _a = this.styles, lineHeight = _a.lineHeight, font = _a.font;
        if (this.canvas) {
            this.canvas.width = this.canvasWidth * dpr;
            this.canvas.height = this.lines * lineHeight * dpr;
        }
        if (!this.ctx)
            return;
        this.ctx.scale(dpr, dpr);
        this.ctx.font = font;
        this.ctx.textBaseline = 'top';
        this.lines = 0;
    };
    Highlighter.prototype.initText = function (text) {
        var _this = this;
        this.calcPosition(text);
        this.resetSize();
        this.highlightInfo.forEach(function (word) {
            _this.drawWord(word);
        });
    };
    Highlighter.prototype.highlightText = function (wordStatus) {
        var _this = this;
        var statusMap = new Map();
        wordStatus.forEach(function (word) {
            statusMap.set(word.index, word.status);
        });
        var newHighlightInfo = [];
        this.highlightInfo.forEach(function (word, index) {
            var status = statusMap.get(index) || exports.HIGHLIGHT_STATUS.NONE;
            newHighlightInfo.push(__assign(__assign({}, word), { status: status }));
            if (status !== word.status)
                _this.drawWord(__assign(__assign({}, word), { status: status }));
        });
        this.highlightInfo = newHighlightInfo;
    };
    Highlighter.prototype.underlineText = function (start, end) {
        var _this = this;
        var newHighlightInfo = [];
        this.highlightInfo.forEach(function (word) {
            var underline = word.index >= start && word.index < end;
            newHighlightInfo.push(__assign(__assign({}, word), { underline: underline }));
            if (underline !== word.underline)
                _this.drawWord(__assign(__assign({}, word), { underline: underline }));
        });
        this.highlightInfo = newHighlightInfo;
    };
    Highlighter.prototype.drawWord = function (word) {
        if (!this.ctx)
            return;
        var _a = this.styles, lineWidth = _a.lineWidth, lineHeight = _a.lineHeight, fontSize = _a.fontSize;
        this.ctx.fillStyle = this.colorMap[word.status];
        this.ctx.save();
        this.ctx.translate(0, (lineHeight - fontSize) / 2);
        this.ctx.clearRect(word.left, word.top, word.width, word.height);
        this.ctx.fillText(word.content, word.left, word.top);
        this.ctx.restore();
        if (!word.underline)
            return;
        var bottom = word.top + lineHeight;
        this.ctx.beginPath();
        this.ctx.strokeStyle = '#26d07c';
        this.ctx.setLineDash([5, 5]);
        this.ctx.lineWidth = lineWidth;
        this.ctx.moveTo(word.left, bottom - lineWidth);
        this.ctx.lineTo(word.left + word.width, bottom - lineWidth);
        this.ctx.stroke();
    };
    return Highlighter;
}());

exports.default = Highlighter;
