export declare enum HIGHLIGHT_STATUS {
    NONE = 0,
    BAD = 1,
    NORMAL = 2,
    GOOD = 3
}
export declare type WordProps = {
    index: number;
    content: string;
    left: number;
    top: number;
    width: number;
    height: number;
    status: HIGHLIGHT_STATUS;
    underline: boolean;
};
export declare type OptionsProps = {
    width: number;
    styles?: Partial<StyleProps>;
    colorMap?: Partial<Record<HIGHLIGHT_STATUS, string>>;
};
export declare type StyleProps = {
    lineHeight: number;
    fontSize: number;
    lineWidth: number;
    font: string;
};
export default class Highlighter {
    canvas: HTMLCanvasElement | null;
    ctx: CanvasRenderingContext2D | null;
    styles: {
        lineHeight: number;
        fontSize: number;
        lineWidth: number;
        font: string;
    };
    colorMap: {
        0: string;
        2: string;
        1: string;
        3: string;
    };
    highlightInfo: WordProps[];
    lines: number;
    canvasWidth: number;
    constructor(container: HTMLCanvasElement | string, options: OptionsProps);
    calcPosition(text: string): void;
    resetSize(): void;
    initText(text: string): void;
    highlightText(wordStatus: {
        index: number;
        status: HIGHLIGHT_STATUS;
    }[]): void;
    underlineText(start: number, end: number): void;
    drawWord(word: WordProps): void;
}
